import os
import cohere
import gradio as gr
from typing import Callable
import base64

__version__ = "0.0.3"


def get_fn(model_name: str, preprocess: Callable, postprocess: Callable, api_key: str):
    def fn(message, history):
        inputs = preprocess(message, history)
        client = cohere.Client(api_key=api_key)
        stream = client.chat_stream(
            model=model_name,
            message=inputs["message"],
            chat_history=inputs["chat_history"],
        )
        response_text = ""
        for chunk in stream:
            if chunk.event_type == "text-generation":
                delta = chunk.text
                response_text += delta
                yield postprocess(response_text)

    return fn


def get_image_base64(url: str, ext: str):
    with open(url, "rb") as image_file:
        encoded_string = base64.b64encode(image_file.read()).decode('utf-8')
    return "data:image/" + ext + ";base64," + encoded_string


def handle_user_msg(message: str):
    if type(message) is str:
        return message
    elif type(message) is dict:
        if message["files"] is not None and len(message["files"]) > 0:
            ext = os.path.splitext(message["files"][-1])[1].strip(".")
            if ext.lower() in ["png", "jpg", "jpeg", "gif", "pdf"]:
                encoded_str = get_image_base64(message["files"][-1], ext)
            else:
                raise NotImplementedError(f"Not supported file type {ext}")
            return {
                "message": message["text"],
                "attachments": [
                    {
                        "source": encoded_str,
                        "type": "image",
                    }
                ]
            }
        else:
            return message["text"]
    else:
        raise NotImplementedError


def get_interface_args(pipeline):
    if pipeline == "chat":
        inputs = None
        outputs = None

        def preprocess(message, history):
            chat_history = []
            files = None
            for user_msg, assistant_msg in history:
                if assistant_msg is not None:
                    chat_history.append({"role": "USER", "message": handle_user_msg(user_msg)})
                    chat_history.append({"role": "ASSISTANT", "message": assistant_msg})
                else:
                    files = user_msg
            if type(message) is str and files is not None:
                message = {"text": message, "files": files}
            elif type(message) is dict and files is not None:
                if message["files"] is None or len(message["files"]) == 0:
                    message["files"] = files
            
            return {
                "message": handle_user_msg(message),
                "chat_history": chat_history
            }

        postprocess = lambda x: x
    else:
        # Add other pipeline types when they will be needed
        raise ValueError(f"Unsupported pipeline type: {pipeline}")
    return inputs, outputs, preprocess, postprocess


def get_pipeline(model_name):
    # Determine the pipeline type based on the model name
    # For simplicity, assuming all models are chat models at the moment
    return "chat"


def registry(name: str, token: str | None = None, **kwargs):
    """
    Create a Gradio Interface for a model on Cohere.

    Parameters:
        - name (str): The name of the Cohere model.
        - token (str, optional): The API key for Cohere.
    """
    api_key = token or os.environ.get("COHERE_API_KEY")
    if not api_key:
        raise ValueError("COHERE_API_KEY environment variable is not set.")

    pipeline = get_pipeline(name)
    inputs, outputs, preprocess, postprocess = get_interface_args(pipeline)
    fn = get_fn(name, preprocess, postprocess, api_key)

    if pipeline == "chat":
        interface = gr.ChatInterface(fn=fn, multimodal=True, **kwargs)
    else:
        # For other pipelines, create a standard Interface (not implemented yet)
        interface = gr.Interface(fn=fn, inputs=inputs, outputs=outputs, **kwargs)

    return interface


import os
import base64
from mistralai import Mistral
import gradio as gr
from typing import Callable
from urllib.parse import urlparse
import modelscope_studio.components.base as ms
import modelscope_studio.components.legacy as legacy
import modelscope_studio.components.antd as antd
import re

__version__ = "0.0.1"

SystemPrompt = """You are an expert web developer specializing in creating clean, efficient, and modern web applications.
Your task is to write complete, self-contained HTML files that include all necessary CSS and JavaScript.
Focus on:
- Writing clear, maintainable code
- Following best practices
- Creating responsive designs
- Adding appropriate styling and interactivity
Return only the complete HTML code without any additional explanation."""


def encode_image_file(image_path):
    """Encode an image file to base64."""
    try:
        with open(image_path, "rb") as image_file:
            return base64.b64encode(image_file.read()).decode('utf-8')
    except Exception as e:
        print(f"Error encoding image: {str(e)}")
        return None


def process_image(image):
    """Process image input to the format expected by Mistral API."""
    if isinstance(image, str):
        # Check if it's a URL or base64 string
        if image.startswith('data:'):
            return image  # Already in base64 format
        elif urlparse(image).scheme in ('http', 'https'):
            return image  # It's a URL
        else:
            # Assume it's a local file path
            encoded = encode_image_file(image)
            return f"data:image/jpeg;base64,{encoded}" if encoded else None
    return None


def get_fn(model_name: str, preprocess: Callable, postprocess: Callable, api_key: str):
    def fn(message, history):
        inputs = preprocess(message, history)
        client = Mistral(api_key=api_key)
        try:
            # Check if the model is Codestral
            if model_name.startswith("codestral"):
                # Handle Codestral API calls
                response = client.chat.complete(
                    model=model_name,
                    messages=inputs["messages"]
                )
                yield postprocess(response.choices[0].message.content)
            else:
                # Create the streaming chat completion for other models
                stream_response = client.chat.stream(
                    model=model_name,
                    messages=inputs["messages"]
                )
                
                response_text = ""
                for chunk in stream_response:
                    if chunk.data.choices[0].delta.content is not None:
                        delta = chunk.data.choices[0].delta.content
                        response_text += delta
                        yield postprocess(response_text)

        except Exception as e:
            print(f"Error during chat completion: {str(e)}")
            yield "Sorry, there was an error processing your request."

    return fn


def get_interface_args(pipeline):
    if pipeline == "chat":
        inputs = None
        outputs = None

        def preprocess(message, history):
            messages = []
            # Process history
            for user_msg, assistant_msg in history:
                if isinstance(user_msg, dict):
                    # Handle multimodal history messages
                    content = []
                    if user_msg.get("text"):
                        content.append({"type": "text", "text": user_msg["text"]})
                    for file in user_msg.get("files", []):
                        processed_image = process_image(file)
                        if processed_image:
                            content.append({"type": "image_url", "image_url": processed_image})
                    messages.append({"role": "user", "content": content})
                else:
                    # Handle text-only history messages
                    messages.append({"role": "user", "content": user_msg})
                messages.append({"role": "assistant", "content": assistant_msg})
            
            # Process current message
            if isinstance(message, dict):
                # Handle multimodal input
                content = []
                if message.get("text"):
                    content.append({"type": "text", "text": message["text"]})
                for file in message.get("files", []):
                    processed_image = process_image(file)
                    if processed_image:
                        content.append({"type": "image_url", "image_url": processed_image})
                messages.append({"role": "user", "content": content})
            else:
                # Handle text-only input
                messages.append({"role": "user", "content": message})
            
            return {"messages": messages}

        postprocess = lambda x: x  # No post-processing needed
    else:
        raise ValueError(f"Unsupported pipeline type: {pipeline}")
    return inputs, outputs, preprocess, postprocess


def get_pipeline(model_name):
    # Determine the pipeline type based on the model name
    # For simplicity, assuming all models are chat models at the moment
    return "chat"


def generate_code(query, history, setting, api_key):
    """Generate code using Mistral API and handle UI updates."""
    client = Mistral(api_key=api_key)
    
    messages = []
    # Add system prompt
    messages.append({"role": "system", "content": setting["system"]})
    
    # Add history
    for h in history:
        messages.append({"role": "user", "content": h[0]})
        messages.append({"role": "assistant", "content": h[1]})
    
    # Add current query
    messages.append({"role": "user", "content": query})
    
    try:
        # Create the streaming chat completion
        stream_response = client.chat.stream(
            model="mistral-large-latest",
            messages=messages
        )
        
        response_text = ""
        for chunk in stream_response:
            if chunk.data.choices[0].delta.content is not None:
                delta = chunk.data.choices[0].delta.content
                response_text += delta
                # Yield intermediate updates
                yield (
                    response_text,          # code_output (for markdown display)
                    history,               # history state
                    None,                  # preview HTML
                    gr.update(active_key="loading"),  # state_tab
                    gr.update(open=True)   # code_drawer
                )
        
        # Clean the code and prepare final preview
        clean_code = remove_code_block(response_text)
        new_history = history + [(query, response_text)]
        
        # Final yield with complete response
        yield (
            response_text,          # code_output
            new_history,           # history state
            send_to_preview(clean_code),  # preview HTML
            gr.update(active_key="render"),  # state_tab
            gr.update(open=False)  # code_drawer
        )
        
    except Exception as e:
        print(f"Error generating code: {str(e)}")
        yield (
            f"Error: {str(e)}",
            history,
            None,
            gr.update(active_key="empty"),
            gr.update(open=True)
        )


def remove_code_block(text):
    """Extract code from markdown code blocks."""
    pattern = r'```html\n(.+?)\n```'
    match = re.search(pattern, text, re.DOTALL)
    if match:
        return match.group(1).strip()
    return text.strip()


def send_to_preview(code):
    """Convert code to base64 encoded iframe source."""
    encoded_html = base64.b64encode(code.encode('utf-8')).decode('utf-8')
    data_uri = f"data:text/html;charset=utf-8;base64,{encoded_html}"
    return f'<iframe src="{data_uri}" width="100%" height="920px"></iframe>'


def registry(name: str, token: str | None = None, coder: bool = False, **kwargs):
    """
    Create a Gradio Interface for a model on Mistral AI.

    Parameters:
        - name (str): The name of the Mistral AI model.
        - token (str, optional): The API key for Mistral AI.
    """
    api_key = token or os.environ.get("MISTRAL_API_KEY")
    if not api_key:
        raise ValueError("MISTRAL_API_KEY environment variable is not set.")

    pipeline = get_pipeline(name)
    inputs, outputs, preprocess, postprocess = get_interface_args(pipeline)
    fn = get_fn(name, preprocess, postprocess, api_key)

    if pipeline == "chat":
        # Always enable multimodal support
        interface = gr.ChatInterface(
            fn=fn,
            multimodal=True,
            **kwargs
        )
    else:
        # For other pipelines, create a standard Interface (not implemented yet)
        interface = gr.Interface(fn=fn, inputs=inputs, outputs=outputs, **kwargs)

    if coder:
        interface = gr.Blocks(css="""
            .left_header {
                text-align: center;
                margin-bottom: 20px;
            }
            .right_panel {
                background: white;
                border-radius: 8px;
                overflow: hidden;
                box-shadow: 0 2px 8px rgba(0,0,0,0.15);
            }
            .render_header {
                background: #f5f5f5;
                padding: 8px;
                border-bottom: 1px solid #e8e8e8;
            }
            .header_btn {
                display: inline-block;
                width: 12px;
                height: 12px;
                border-radius: 50%;
                margin-right: 8px;
                background: #ff5f56;
            }
            .header_btn:nth-child(2) {
                background: #ffbd2e;
            }
            .header_btn:nth-child(3) {
                background: #27c93f;
            }
            .right_content {
                padding: 24px;
                height: 920px;
                display: flex;
                align-items: center;
                justify-content: center;
            }
            .html_content {
                height: 920px;
                width: 100%;
            }
            .history_chatbot {
                height: 100%;
            }
        """)

        with interface:
            history = gr.State([])
            setting = gr.State({"system": SystemPrompt})

            with ms.Application() as app:
                with antd.ConfigProvider():
                    with antd.Row(gutter=[32, 12]) as layout:
                        # Left Column
                        with antd.Col(span=24, md=8):
                            with antd.Flex(vertical=True, gap="middle", wrap=True):
                                header = gr.HTML("""
                                    <div class="left_header">
                                        <h1>Codestral Code Generator</h1>
                                    </div>
                                """)
                                
                                input = antd.InputTextarea(
                                    size="large",
                                    allow_clear=True,
                                    placeholder="Describe the code you want to generate"
                                )
                                btn = antd.Button("Generate", type="primary", size="large")
                                clear_btn = antd.Button("Clear History", type="default", size="large")

                                antd.Divider("Settings")
                                with antd.Flex(gap="small", wrap=True):
                                    settingPromptBtn = antd.Button("⚙️ System Prompt", type="default")
                                    codeBtn = antd.Button("🧑‍💻 View Code", type="default")
                                    historyBtn = antd.Button("📜 History", type="default")

                        # Modals and Drawers
                        with antd.Modal(open=False, title="System Prompt", width="800px") as system_prompt_modal:
                            systemPromptInput = antd.InputTextarea(SystemPrompt, auto_size=True)

                        with antd.Drawer(open=False, title="Code", placement="left", width="750px") as code_drawer:
                            code_output = legacy.Markdown()

                        with antd.Drawer(open=False, title="History", placement="left", width="900px") as history_drawer:
                            history_output = legacy.Chatbot(
                                show_label=False,
                                height=960,
                                elem_classes="history_chatbot"
                            )

                        # Right Column
                        with antd.Col(span=24, md=16):
                            with ms.Div(elem_classes="right_panel"):
                                gr.HTML('''
                                    <div class="render_header">
                                        <span class="header_btn"></span>
                                        <span class="header_btn"></span>
                                        <span class="header_btn"></span>
                                    </div>
                                ''')
                                with antd.Tabs(active_key="empty", render_tab_bar="() => null") as state_tab:
                                    with antd.Tabs.Item(key="empty"):
                                        empty = antd.Empty(
                                            description="Enter your request to generate code",
                                            elem_classes="right_content"
                                        )
                                    with antd.Tabs.Item(key="loading"):
                                        loading = antd.Spin(
                                            True,
                                            tip="Generating code...",
                                            size="large",
                                            elem_classes="right_content"
                                        )
                                    with antd.Tabs.Item(key="render"):
                                        preview = gr.HTML(elem_classes="html_content")

            # Wire up event handlers
            btn.click(
                generate_code,
                inputs=[input, history, setting, gr.State(api_key)],
                outputs=[code_output, history, preview, state_tab, code_drawer],
                api_name=False
            )
            
            settingPromptBtn.click(lambda: gr.update(open=True), outputs=[system_prompt_modal])
            system_prompt_modal.ok(
                lambda input: ({"system": input}, gr.update(open=False)),
                inputs=[systemPromptInput],
                outputs=[setting, system_prompt_modal]
            )
            system_prompt_modal.cancel(lambda: gr.update(open=False), outputs=[system_prompt_modal])
            
            codeBtn.click(lambda: gr.update(open=True), outputs=[code_drawer])
            code_drawer.close(lambda: gr.update(open=False), outputs=[code_drawer])
            
            historyBtn.click(
                lambda h: (gr.update(open=True), h),
                inputs=[history],
                outputs=[history_drawer, history_output]
            )
            history_drawer.close(lambda: gr.update(open=False), outputs=[history_drawer])
            
            clear_btn.click(lambda: [], outputs=[history])

    return interface


import os
from openai import OpenAI
import gradio as gr
from typing import Callable
import re
import base64
import modelscope_studio.components.base as ms
import modelscope_studio.components.legacy as legacy
import modelscope_studio.components.antd as antd

__version__ = "0.0.1"

# Add these constants at the top of the file
SystemPrompt = """You are an expert web developer specializing in creating clean, efficient, and modern web applications.
Your task is to write complete, self-contained HTML files that include all necessary CSS and JavaScript.
Focus on:
- Writing clear, maintainable code
- Following best practices
- Creating responsive designs
- Adding appropriate styling and interactivity
Return only the complete HTML code without any additional explanation."""

DEMO_LIST = [
    {
        "card": {"index": 0},
        "title": "Simple Button",
        "description": "Create a button that changes color when clicked"
    },
    {
        "card": {"index": 1},
        "title": "Todo List",
        "description": "Create a simple todo list with add/remove functionality"
    },
    {
        "card": {"index": 2},
        "title": "Timer App",
        "description": "Create a countdown timer with start/pause/reset controls"
    }
]


def get_fn(model_name: str, preprocess: Callable, postprocess: Callable, api_key: str):
    def fn(message, history):
        inputs = preprocess(message, history)
        client = OpenAI(
            api_key=api_key,
            base_url="https://api.perplexity.ai"
        )
        completion = client.chat.completions.create(
            model=model_name,
            messages=inputs["messages"],
            stream=True,
        )
        response_text = ""
        for chunk in completion:
            delta = chunk.choices[0].delta.content or ""
            response_text += delta
            yield postprocess(response_text)

    return fn


def get_interface_args(pipeline):
    if pipeline == "chat":
        inputs = None
        outputs = None

        def preprocess(message, history):
            messages = []
            for user_msg, assistant_msg in history:
                messages.append({"role": "user", "content": user_msg})
                messages.append({"role": "assistant", "content": assistant_msg})
            messages.append({"role": "user", "content": message})
            return {"messages": messages}

        postprocess = lambda x: x  # No post-processing needed
    else:
        # Add other pipeline types when they will be needed
        raise ValueError(f"Unsupported pipeline type: {pipeline}")
    return inputs, outputs, preprocess, postprocess


def get_pipeline(model_name):
    # Determine the pipeline type based on the model name
    # For simplicity, assuming all models are chat models at the moment
    return "chat"


def registry(
    name: str, 
    token: str | None = None,
    examples: list | None = None,
    coder: bool = False,  # Add coder parameter
    **kwargs
):
    """
    Create a Gradio Interface for a model on Perplexity.

    Parameters:
        - name (str): The name of the model
        - token (str, optional): The API key
        - examples (list, optional): Example inputs
        - coder (bool, optional): Whether to use coding interface
    """
    api_key = token or os.environ.get("PERPLEXITY_API_KEY")
    if not api_key:
        raise ValueError("PERPLEXITY_API_KEY environment variable is not set.")

    if coder:
        interface = gr.Blocks(css="""
            .left_header {
                text-align: center;
                margin-bottom: 20px;
            }

            .right_panel {
                background: white;
                border-radius: 8px;
                overflow: hidden;
                box-shadow: 0 2px 8px rgba(0,0,0,0.15);
            }

            .render_header {
                background: #f5f5f5;
                padding: 8px;
                border-bottom: 1px solid #e8e8e8;
            }

            .header_btn {
                display: inline-block;
                width: 12px;
                height: 12px;
                border-radius: 50%;
                margin-right: 8px;
                background: #ff5f56;
            }

            .header_btn:nth-child(2) {
                background: #ffbd2e;
            }

            .header_btn:nth-child(3) {
                background: #27c93f;
            }

            .right_content {
                padding: 24px;
                height: 920px;
                display: flex;
                align-items: center;
                justify-content: center;
            }

            .html_content {
                height: 920px;
                width: 100%;
            }

            .history_chatbot {
                height: 100%;
            }
        """)
        with interface:
            history = gr.State([])
            setting = gr.State({"system": SystemPrompt})

            with ms.Application() as app:
                with antd.ConfigProvider():
                    with antd.Row(gutter=[32, 12]) as layout:
                        # Left Column
                        with antd.Col(span=24, md=8):
                            with antd.Flex(vertical=True, gap="middle", wrap=True):
                                header = gr.HTML("""
                                    <div class="left_header">
                                        <h1>Perplexity Code Generator</h1>
                                    </div>
                                """)
                                
                                input = antd.InputTextarea(
                                    size="large",
                                    allow_clear=True,
                                    placeholder="Describe the web application you want to create"
                                )
                                btn = antd.Button("Generate", type="primary", size="large")
                                clear_btn = antd.Button("Clear History", type="default", size="large")

                                antd.Divider("Examples")
                                with antd.Flex(gap="small", wrap=True):
                                    with ms.Each(DEMO_LIST):
                                        with antd.Card(hoverable=True, as_item="card") as demoCard:
                                            antd.CardMeta()
                                
                                antd.Divider("Settings")
                                with antd.Flex(gap="small", wrap=True):
                                    settingPromptBtn = antd.Button("⚙️ System Prompt", type="default")
                                    codeBtn = antd.Button("🧑‍💻 View Code", type="default")
                                    historyBtn = antd.Button("📜 History", type="default")

                            # Modals and Drawers
                            with antd.Modal(open=False, title="System Prompt", width="800px") as system_prompt_modal:
                                systemPromptInput = antd.InputTextarea(SystemPrompt, auto_size=True)

                            with antd.Drawer(open=False, title="Code", placement="left", width="750px") as code_drawer:
                                code_output = legacy.Markdown()

                            with antd.Drawer(open=False, title="History", placement="left", width="900px") as history_drawer:
                                history_output = legacy.Chatbot(
                                    show_label=False,
                                    height=960,
                                    elem_classes="history_chatbot"
                                )

                        # Right Column
                        with antd.Col(span=24, md=16):
                            with ms.Div(elem_classes="right_panel"):
                                gr.HTML('''
                                    <div class="render_header">
                                        <span class="header_btn"></span>
                                        <span class="header_btn"></span>
                                        <span class="header_btn"></span>
                                    </div>
                                ''')
                                with antd.Tabs(active_key="empty", render_tab_bar="() => null") as state_tab:
                                    with antd.Tabs.Item(key="empty"):
                                        empty = antd.Empty(
                                            description="Enter your request to generate code",
                                            elem_classes="right_content"
                                        )
                                    with antd.Tabs.Item(key="loading"):
                                        loading = antd.Spin(
                                            True,
                                            tip="Generating code...",
                                            size="large",
                                            elem_classes="right_content"
                                        )
                                    with antd.Tabs.Item(key="render"):
                                        preview = gr.HTML(elem_classes="html_content")

            # Event Handlers
            def demo_card_click(e: gr.EventData):
                index = e._data['component']['index']
                return DEMO_LIST[index]['description']

            def send_to_preview(code):
                encoded_html = base64.b64encode(code.encode('utf-8')).decode('utf-8')
                data_uri = f"data:text/html;charset=utf-8;base64,{encoded_html}"
                return f'<iframe src="{data_uri}" width="100%" height="920px"></iframe>'

            def remove_code_block(text):
                # Look for HTML code block first
                html_pattern = r'```html\n(.*?)\n```'
                match = re.search(html_pattern, text, re.DOTALL)
                if match:
                    return match.group(1).strip()
                
                # If no HTML block found, look for any code block
                code_pattern = r'```(?:\w+)?\n(.*?)\n```'
                match = re.search(code_pattern, text, re.DOTALL)
                if match:
                    return match.group(1).strip()
                
                # If no code block found, return the whole text
                return text.strip()

            def generate_code(query, setting, history):
                messages = []
                messages.append({"role": "system", "content": setting["system"]})
                
                for h in history:
                    messages.append({"role": "user", "content": h[0]})
                    messages.append({"role": "assistant", "content": h[1]})
                
                messages.append({"role": "user", "content": query})
                
                client = OpenAI(
                    api_key=api_key,
                    base_url="https://api.perplexity.ai"
                )
                
                response = client.chat.completions.create(
                    model=name,
                    messages=messages,
                    stream=True
                )
                
                response_text = ""
                for chunk in response:
                    if chunk.choices[0].delta.content:
                        response_text += chunk.choices[0].delta.content
                        # Format the code for display
                        formatted_response = f"```html\n{response_text}\n```"
                        # Return all 5 required outputs
                        yield (
                            formatted_response,  # code_output (modelscopemarkdown)
                            history,            # state
                            None,              # preview (html)
                            gr.update(active_key="loading"),  # state_tab (antdtabs)
                            gr.update(open=True)  # code_drawer (antddrawer)
                        )
                
                clean_code = remove_code_block(response_text)
                new_history = history + [(query, response_text)]
                
                # Final yield with all outputs
                yield (
                    f"```html\n{clean_code}\n```",  # code_output (modelscopemarkdown)
                    new_history,    # state
                    send_to_preview(clean_code),  # preview (html)
                    gr.update(active_key="render"),  # state_tab (antdtabs)
                    gr.update(open=False)  # code_drawer (antddrawer)
                )

            # Wire up event handlers
            demoCard.click(demo_card_click, outputs=[input])
            settingPromptBtn.click(lambda: gr.update(open=True), outputs=[system_prompt_modal])
            system_prompt_modal.ok(
                lambda input: ({"system": input}, gr.update(open=False)),
                inputs=[systemPromptInput],
                outputs=[setting, system_prompt_modal]
            )
            system_prompt_modal.cancel(lambda: gr.update(open=False), outputs=[system_prompt_modal])
            
            codeBtn.click(lambda: gr.update(open=True), outputs=[code_drawer])
            code_drawer.close(lambda: gr.update(open=False), outputs=[code_drawer])
            
            historyBtn.click(
                lambda h: (gr.update(open=True), h),
                inputs=[history],
                outputs=[history_drawer, history_output]
            )
            history_drawer.close(lambda: gr.update(open=False), outputs=[history_drawer])
            
            btn.click(
                generate_code,
                inputs=[input, setting, history],
                outputs=[code_output, history, preview, state_tab, code_drawer]
            )
            
            clear_btn.click(lambda: [], outputs=[history])

        return interface

    # Continue with existing chat interface code...
    pipeline = get_pipeline(name)
    inputs, outputs, preprocess, postprocess = get_interface_args(pipeline)
    fn = get_fn(name, preprocess, postprocess, api_key)

    if examples:
        kwargs["examples"] = examples

    if pipeline == "chat":
        interface = gr.ChatInterface(fn=fn, **kwargs)
    else:
        interface = gr.Interface(fn=fn, inputs=inputs, outputs=outputs, **kwargs)

    return interface


import json
import os
import uuid
from collections.abc import Callable
from contextlib import suppress

import gradio as gr
import requests
from openai import OpenAI

__version__ = "0.0.1"

kNvcfAssetUrl = "https://api.nvcf.nvidia.com/v2/nvcf/assets"
kSupportedList = {
    "png": ["image/png", "img"],
    "jpg": ["image/jpg", "img"],
    "jpeg": ["image/jpeg", "img"],
    "mp4": ["video/mp4", "video"],
}


def get_extention(filename):
    _, ext = os.path.splitext(filename)
    ext = ext[1:].lower()
    return ext


def mime_type(ext):
    return kSupportedList[ext][0]


def media_type(ext):
    return kSupportedList[ext][1]


def _upload_asset(media_file, api_key, description):
    ext = get_extention(media_file)
    assert ext in kSupportedList
    data_input = open(media_file, "rb")
    headers = {
        "Authorization": f"Bearer {api_key}",
        "Content-Type": "application/json",
        "accept": "application/json",
    }
    assert_url = kNvcfAssetUrl
    authorize = requests.post(
        assert_url,
        headers=headers,
        json={"contentType": f"{mime_type(ext)}", "description": description},
        timeout=30,
    )
    authorize.raise_for_status()

    authorize_res = authorize.json()
    response = requests.put(
        authorize_res["uploadUrl"],
        data=data_input,
        headers={
            "x-amz-meta-nvcf-asset-description": description,
            "content-type": mime_type(ext),
        },
        timeout=300,
    )

    response.raise_for_status()
    return uuid.UUID(authorize_res["assetId"])


def _delete_asset(asset_id, api_key):
    headers = {
        "Authorization": f"Bearer {api_key}",
    }
    assert_url = f"{kNvcfAssetUrl}/{asset_id}"
    response = requests.delete(assert_url, headers=headers, timeout=30)
    response.raise_for_status()


def chat_with_media_nvcf(infer_url, media_files, query: str, stream: bool = False, api_key: str = None):
    asset_list = []
    ext_list = []
    media_content = ""

    # Handle case when media_files is None
    media_files = media_files or []
    if not isinstance(media_files, list):
        media_files = [media_files]

    has_video = False
    for media_file in media_files:
        if media_file is None:  # Skip if no file
            continue
        ext = get_extention(media_file)
        assert ext in kSupportedList, f"{media_file} format is not supported"
        if media_type(ext) == "video":
            has_video = True
        asset_id = _upload_asset(media_file, api_key, "Reference media file")
        asset_list.append(f"{asset_id}")
        ext_list.append(ext)
        media_content += f'<{media_type(ext)} src="data:{mime_type(ext)};asset_id,{asset_id}" />'

    if has_video:
        assert len(media_files) == 1, "Only single video supported."

    asset_seq = ",".join(asset_list)
    headers = {
        "Authorization": f"Bearer {api_key}",
        "Content-Type": "application/json",
        "NVCF-INPUT-ASSET-REFERENCES": asset_seq,
        "NVCF-FUNCTION-ASSET-IDS": asset_seq,
        "Accept": "application/json",
    }
    if stream:
        headers["Accept"] = "text/event-stream"
    response = None

    messages = [
        {
            "role": "user",
            "content": f"{query} {media_content}",
        }
    ]
    payload = {
        "max_tokens": 1024,
        "temperature": 0.2,
        "top_p": 0.7,
        "seed": 50,
        "num_frames_per_inference": 8,
        "messages": messages,
        "stream": stream,
        "model": "nvidia/vila",
    }

    try:
        response = requests.post(infer_url, headers=headers, json=payload, stream=stream)
        response.raise_for_status()

        if stream:
            output = ""
            for line in response.iter_lines():
                if line:
                    line_str = line.decode("utf-8")
                    if line_str == "data: [DONE]":
                        break
                    data = json.loads(line_str[6:])
                    content = data["choices"][0]["delta"]["content"]
                    if content:
                        output += content
                    yield output
        else:
            yield response.json()["choices"][0]["message"]["content"]
    except Exception as e:
        yield f"Error: {e!s}"
    finally:
        # Clean up assets
        for asset_id in asset_list:
            with suppress(Exception):
                _delete_asset(asset_id, api_key)


def get_fn(model_name: str, preprocess: Callable, postprocess: Callable, api_key: str):
    if model_name == "nvidia/cosmos-nemotron-34b":  # VLM model

        def fn(message, history):
            if history:
                gr.Warning("This app does not support multi-turn conversation.")
                yield "Error"
                return
            if not message["files"]:
                gr.Warning("Please upload either one video or 1-16 images.")
                yield "Error"
                return

            yield from chat_with_media_nvcf(
                "https://ai.api.nvidia.com/v1/vlm/nvidia/cosmos-nemotron-34b",
                message["files"],
                message["text"],
                stream=True,
                api_key=api_key,
            )

        return fn

    else:  # Regular NIM model
        def fn(message, history):
            inputs = preprocess(message, history)
            client = OpenAI(base_url="https://integrate.api.nvidia.com/v1", api_key=api_key)
            completion = client.chat.completions.create(
                model=model_name,
                messages=inputs["messages"],
                temperature=0.2,
                top_p=0.7,
                max_tokens=1024,
                stream=True,
            )
            response_text = ""
            for chunk in completion:
                if chunk.choices[0].delta.content is not None:
                    delta = chunk.choices[0].delta.content
                    response_text += delta
                    yield postprocess(response_text)

        return fn


def get_interface_args(pipeline, model_name):
    if pipeline == "chat":
        inputs = None
        outputs = None

        def preprocess(message, history):
            messages = []
            for user_msg, assistant_msg in history:
                messages.append({"role": "user", "content": str(user_msg)})
                messages.append({"role": "assistant", "content": str(assistant_msg)})
            
            # Handle both string and dictionary message formats
            if isinstance(message, dict):
                message = message.get('text', '')
            messages.append({"role": "user", "content": str(message)})
            return {"messages": messages}

        postprocess = lambda x: x  # No post-processing needed
    else:
        raise ValueError(f"Unsupported pipeline type: {pipeline}")
    return inputs, outputs, preprocess, postprocess


def get_pipeline(model_name):
    # Determine the pipeline type based on the model name
    # For simplicity, assuming all models are chat models at the moment
    return "chat"


def registry(name: str, token: str | None = None, **kwargs):
    """Create a Gradio Interface for a model on NVIDIA NIM or VLM.

    Parameters:
        - name (str): The name of the NVIDIA model (e.g. "nvidia/vila" for VLM).
        - token (str, optional): The API key for NVIDIA services.

    """
    if "cosmos-nemotron-34b" in name:  # VLM model
        api_key = token or os.environ.get("TEST_NVCF_API_KEY")
        if not api_key:
            raise ValueError("TEST_NVCF_API_KEY environment variable is not set.")
    else:
        api_key = token or os.environ.get("NVIDIA_API_KEY")
        if not api_key:
            raise ValueError("NVIDIA_API_KEY environment variable is not set.")

    pipeline = get_pipeline(name)
    inputs, outputs, preprocess, postprocess = get_interface_args(pipeline, name)
    fn = get_fn(name, preprocess, postprocess, api_key)

    if pipeline == "chat":
        if "cosmos-nemotron-34b" in name:
            kwargs["type"] = "messages"
            kwargs["textbox"] = gr.MultimodalTextbox(
                file_count="multiple", file_types=[".png", ".jpg", ".jpeg", ".mp4"]
            )
            interface = gr.ChatInterface(fn=fn, **kwargs)
        else:
            interface = gr.ChatInterface(fn=fn, multimodal=True, **kwargs)
    else:
        interface = gr.Interface(fn=fn, inputs=inputs, outputs=outputs, **kwargs)

    return interface
